@@ -27,6 +27,13 @@
 #include <QSet>
 #include <QStringList>
 
+#include <QDBusMessage>
+#include <QDBusError>
+#include <QDBusContext>
+#include <QDBusObjectPath>
+
+#include <QSocketNotifier>
+
 #include <KSharedConfig>
 
 namespace KActivities
@@ -48,7 +55,14 @@ namespace PowerDevil
 class BackendInterface;
 class Action;
 
-class Q_DECL_EXPORT Core : public QObject
+struct WakeupInfo {
+    QString service;
+    QDBusObjectPath path;
+    int cookie;
+    qint64 timeout;
+};
+
+class Q_DECL_EXPORT Core : public QObject, protected QDBusContext
 {
     Q_OBJECT
     Q_DISABLE_COPY(Core)
@@ -86,6 +100,16 @@ public Q_SLOTS:
     bool isLidPresent() const;
     bool isActionSupported(const QString &actionName);
     bool hasDualGpu() const;
+    int chargeStartThreshold() const;
+    int chargeStopThreshold() const;
+
+    // service - dbus interface to ping when wakeup is done
+    // path - dbus path on service
+    // cookie - data to pass back
+    // silent - true if silent wakeup is needed
+    uint scheduleWakeup(const QString &service, const QDBusObjectPath &path, qint64 timeout);
+    void wakeup();
+    void clearWakeup(int cookie);
 
 Q_SIGNALS:
     void coreReady();
@@ -93,6 +117,8 @@ Q_SIGNALS:
     void configurationReloaded();
     void batteryRemainingTimeChanged(qulonglong time);
     void lidClosedChanged(bool closed);
+    void chargeStartThresholdChanged(int threshold);
+    void chargeStopThresholdChanged(int threshold);
 
 private:
     void registerActionTimeout(Action *action, int timeout);
@@ -100,6 +126,8 @@ private:
     void handleLowBattery(int percent);
     void handleCriticalBattery(int percent);
 
+    void readChargeThreshold();
+
     /**
      * Computes the current global charge percentage.
      * Sum of all battery charges.
@@ -109,6 +137,8 @@ private:
     friend class Action;
 
     bool m_hasDualGpu;
+    int m_chargeStartThreshold = 0;
+    int m_chargeStopThreshold = 100;
 
     BackendInterface *m_backend;
 
@@ -134,6 +164,12 @@ private:
     QSet<Action *> m_pendingResumeFromIdleActions;
     bool m_pendingWakeupEvent;
 
+    // Scheduled wakeups and alarms
+    QList<WakeupInfo> m_scheduledWakeups;
+    int m_lastWakeupCookie = 0;
+    int m_timerFd = -1;
+    QSocketNotifier *m_timerFdSocketNotifier = nullptr;
+
     // Activity inhibition management
     QHash< QString, int > m_sessionActivityInhibit;
     QHash< QString, int > m_screenActivityInhibit;
@@ -153,6 +189,9 @@ private Q_SLOTS:
     void onServiceRegistered(const QString &service);
     void onLidClosedChanged(bool closed);
     void onAboutToSuspend();
+    // handlers for handling wakeup dbus call
+    void resetAndScheduleNextWakeup();
+    void timerfdEventHandler();
 };
 
 }
